from rest_framework.routers import SimpleRouter
from django.urls import path

from . import views

app_name = 'drf_schemas'

router = SimpleRouter()

router.register(r'boolean-fields', views.BooleanFieldView, 'boolean-fields')
router.register(r'choices-fields', views.ChoicesFieldView, 'choices-fields')
router.register(r'datetime-fields', views.DateTimeFieldView, 'datetime-fields')
router.register(r'file-fields', views.FileFieldView, 'file-fields')
router.register(r'images-fields', views.ImagesFieldView, 'images-fields')
router.register(r'item-fields', views.ItemFieldView, 'item-fields')
router.register(r'number-fields', views.NumberFieldView, 'number-fields')
router.register(r'text-fields', views.TextFieldView, 'text-fields')

router.register(r'boolean-values', views.BooleanValueView, 'bool-values')
router.register(r'choices-values', views.ChoicesValueView, 'choices-values')
router.register(r'datetime-values', views.DateTimeValueView, 'datetime-values')
router.register(r'file-values', views.FileValueView, 'file-values')
router.register(r'images-values', views.ImagesValueView, 'images-values')
router.register(r'item-values', views.ItemValueView, 'item-values')
router.register(r'number-values', views.NumberValueView, 'number-values')
router.register(r'text-values', views.TextValueView, 'text-values')

router.register(r'choices', views.ChoiceView, 'choices')
router.register(r'files', views.FileView, 'files')
router.register(r'images', views.ImageView, 'images')

router.register(r'categories', views.CategoryView, 'categories')
router.register(r'item-schemas', views.ItemSchemaView, 'item-schemas')
router.register(r'items', views.ItemView, 'items')

urlpatterns = router.urls + [
    path('export/', views.ExportView.as_view(), name='export'),
    path('import/', views.ImportView.as_view(), name='import')
]



from rest_framework import viewsets

from ._mixins import (
    CreateMixin,
    ListMixin,
    RetrieveMixin,
    DestroyMixin,
    UpdateMixin,
    FilterMixin
)
from ..models import (
    Item,
    ItemSchema,
    Category
)
from ..serializers import (
    ItemSerializer,
    ItemSchemaSerializer,
    CategorySerializer,
    ItemFilterSerializer,
    ItemSchemaFilterSerializer
)


class ItemView(
    CreateMixin,
    ListMixin,
    RetrieveMixin,
    DestroyMixin,
    UpdateMixin,
    FilterMixin,
    viewsets.GenericViewSet
):

    lookup_field = 'pk'
    model_name = 'Item'
    queryset = Item.objects.all()
    serializer_class = ItemSerializer
    filter_serializer_class = ItemFilterSerializer


class CategoryView(
    CreateMixin,
    ListMixin,
    RetrieveMixin,
    DestroyMixin,
    UpdateMixin,
    viewsets.GenericViewSet
):

    lookup_field = 'pk'
    model_name = 'Category'
    queryset = Category.objects.all()
    serializer_class = CategorySerializer

    def get_queryset(self):
        queryset = self.queryset

        name = self.request.query_params.get('name', '')
        if name:
            queryset = queryset.filter(name__icontains=name)

        order_by = self.request.query_params.get('order_by', '')
        if order_by in ['name', 'created_at', 'updated_at']:
            queryset = queryset.order_by(order_by)

        return queryset


class ItemSchemaView(ItemView):

    model_name = 'ItemSchema'
    queryset = ItemSchema.objects.all()
    serializer_class = ItemSchemaSerializer
    filter_serializer_class = ItemSchemaFilterSerializer





import json
from datetime import datetime

from django.http import HttpResponse
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response
from rest_framework.views import APIView

from ..serializers import ImportSerializer
from ..services import export_schemas, import_schemas

JSON_MIME = 'application/json'
JSON_NAME = 'attachment; filename="schemas-{}.json"'


class ExportView(APIView):

    def get(self, request):
        params = self.request.query_params

        now = datetime.now().strftime('%Y%m%d-%H%M')
        response = HttpResponse(content_type=JSON_MIME)
        response['Content-Disposition'] = JSON_NAME.format(now)

        data = export_schemas()
        json.dump(data, response)

        return response


class ImportView(APIView):

    def post(self, request):
        serializer = ImportSerializer(
            data=request.data,
            context={'request': request}
        )
        serializer.is_valid(raise_exception=True)
        file = serializer.validated_data['file']
        try:
            data = json.load(file)
        except Exception as error:
            raise ValidationError(error)
        else:
            errors = import_schemas(data)
            if errors:
                raise ValidationError(errors)

        return Response(status=status.HTTP_201_CREATED)
